package com.fs.store.controller;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.fs.FSAdminApplication;
import com.fs.common.core.domain.R;
import com.fs.common.exception.CustomException;
import com.fs.erp.dto.*;
import com.fs.erp.http.JstErpHttpService;
import com.fs.erp.service.FsJstAftersalePushService;
import com.fs.erp.service.IErpGoodsService;
import com.fs.erp.service.IErpOrderService;
import com.fs.erp.utils.SignUtil;
import com.fs.huifuPay.dto.*;
import com.fs.pay.pay.domain.OrderResult;
import com.fs.pay.pay.dto.OrderQueryDTO;
import com.fs.pay.pay.service.PayService;
import com.fs.store.domain.*;
import com.fs.store.enums.IcgProcessStatusEnum;
import com.fs.store.mapper.FsMiniprogramSubNotifyTaskMapper;
import com.fs.store.mapper.FsStoreDeliversMapper;
import com.fs.store.mapper.FsStoreOrderMapper;
import com.fs.store.mapper.FsStorePaymentMapper;
import com.fs.store.service.IFsCouponScheduleService;
import com.fs.store.service.IFsStoreOrderService;
import com.fs.store.service.IFsStorePaymentService;
import com.fs.store.service.channel.HfPaymentHandler;
import com.fs.store.service.channel.YbPaymentHandler;
import com.fs.store.service.channel.param.PayProcessContext;
import com.fs.system.service.ISysConfigService;
import com.fs.task.ExpressTask;
import com.fs.task.MiniProgramSubTask;
import com.fs.task.StoreTask;
import com.fs.tzBank.TzBankService;
import com.fs.tzBank.utils.TzConfigUtils;
import com.fs.wx.domain.FsWxExpressTask;
import com.fs.wx.mapper.FsWxExpressTaskMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = FSAdminApplication.class)
@RequiredArgsConstructor
@Slf4j
public class FsStorePaymentControllerTest {

    @Autowired
    private FsStorePaymentController fsStorePaymentController;

    @Autowired
    private IFsCouponScheduleService fsCouponScheduleService;

    @Autowired
    IFsStorePaymentService fsStorePaymentService;

    @Autowired
    FsStorePaymentMapper fsStorePaymentMapper;

    @Autowired
    PayService payService;

    @Autowired
    private FsWxExpressTaskMapper fsWxExpressTaskMapper;

    @Autowired
    private FsStoreDeliversMapper fsStoreDeliversMapper;

    @Autowired
    private FsStoreOrderMapper fsStoreOrderMapper;

    @Autowired
    private StoreTask storeTask;


    @Autowired
    private JstErpHttpService httpService;

    @Autowired
    private IFsStoreOrderService fsStoreOrderService;

    @Autowired
    private IErpOrderService erpOrderService;

    @Autowired
    private IErpGoodsService erpGoodsService;

    @Autowired
    private FsJstAftersalePushService fsJstAftersalePushService;

    @Test
    public void pushJstTest(){
        fsJstAftersalePushService.pushJst();
    }
    @Test
    public void updateExpress(){
        storeTask.updateExpress();
    }

    @Test
    public void pushErp() throws ParseException {
        fsStoreOrderService.createOmsOrder(1332687L);
    }

    @Test
    public void getErpInfo() {
        ErpOrderQueryRequert request = new ErpOrderQueryRequert();
        request.setCode("28377458");
        ErpOrderQueryResponse order = erpOrderService.getOrder(request);
        log.info("订单信息: {}",order);
    }

    @Test
    public void getGoods(){
        ErpGoodsQueryRequert erpGoodsQueryRequert = new ErpGoodsQueryRequert();
        erpGoodsQueryRequert.setCode("105220");
        ErpGoodsQueryResponse goods = erpGoodsService.getGoods(erpGoodsQueryRequert);

        log.info("查询结果: {}",goods);
    }
    @Test
    public void initJstToken() {
        GetInitTokenRequestDTO requestDTO = new GetInitTokenRequestDTO();
        requestDTO.setApp_key("bd7a7d06a6d54377842f883f9a4c502a");
        requestDTO.setGrant_type("authorization_code");
        requestDTO.setCharset("utf-8");
        requestDTO.setCode("zzzzzz");
        requestDTO.setTimestamp(String.valueOf(System.currentTimeMillis() / 1000));

        requestDTO.setSign(SignUtil.getSignNew("1d112cb9d21d41e187ff824b5852f4cf",requestDTO));

        log.info("请求参数: {}", JSON.toJSONString(requestDTO));
        GetInitTokenResponseDTO initToken = httpService.getInitToken(requestDTO);
        log.info(JSON.toJSONString(initToken));
    }


    /**
     * 补事务id
     */
    @Test
    public void fixTransactionNull(){
        List<FsStorePayment> list = fsStorePaymentMapper.queryNormalData();
        for (FsStorePayment fsStorePayment : list) {
            String json = configService.selectConfigByKey("store.pay");
            FsPayConfig fsPayConfig = JSON.parseObject(json, FsPayConfig.class);
            OrderQueryDTO orderQueryDTO = new OrderQueryDTO();
            orderQueryDTO.setAccount(fsPayConfig.getYbAccount());
            orderQueryDTO.setUpOrderId(fsStorePayment.getTradeNo());
            OrderResult order = payService.getOrder(orderQueryDTO);
            if(order != null) {
                fsStorePayment.setBankTransactionId(order.getBankTrxId());
                fsStorePayment.setBankSerialNo(order.getBankOrderId());
            }

            fsStorePaymentMapper.updateFsStorePayment(fsStorePayment);
        }
    }

    /**
     * 修复微信发货
     */
    @Test
    public void fixWxExpress(){
        List<String> bankTrxIds = Arrays.asList(
                "4200002641202507015854132060",
                "4200002730202507010585137790",
                "4200002720202507011716728434",
                "4200002640202506305334957109",
                "4200002649202506308389926327",
                "4200002744202506301134168611",
                "4200002657202506305253885830",
                "4200002742202506294065544939",
                "4200002646202506290678964674",
                "4200002730202506290169087232",
                "4200002639202506295681468881",
                "4200002663202506295495889025",
                "4200002649202506291445021052",
                "4200002744202506280680760540",
                "4200002635202506284191091408",
                "4200002656202506285273335101",
                "4200002743202506288914905137",
                "4200002643202506286623634610",
                "4200002743202506282965581155",
                "4200002737202506287558895883",
                "4200002736202506251277657543",
                "4200002735202506253277037320",
                "4200002652202506254837674288",
                "4200002645202506251296123074",
                "4200002644202506256761292850",
                "4200002641202506252183156757",
                "4200002742202506241338120931",
                "4200002744202506249664154866",
                "4200002663202506246584120213",
                "4200002653202506245360228516",
                "4200002721202506247137648457",
                "4200002646202506239575111341",
                "4200002738202506230718968779",
                "4200002736202506236962272488",
                "4200002733202506230550173916",
                "4200002738202506232322453502",
                "4200002725202506239319858501",
                "4200002634202506225400496329",
                "4200002738202506210520565498",
                "4200002729202506214580870247",
                "4200002740202506210566404836",
                "4200002667202506204390460462",
                "4200002719202506202245237392",
                "4200002651202506202505391109",
                "4200002656202506200517612411",
                "4200002722202506203205725652",
                "4200002638202506209508809661",
                "4200002634202506201882461115",
                "4200002651202506204904211840",
                "4200002644202506195185880011",
                "4200002666202506195309219440",
                "4200002663202506191821261470",
                "4200002742202506197219115356",
                "4200002661202506192063355144",
                "4200002644202506192082596482",
                "4200002743202506193920004348",
                "4200002743202506194808329574",
                "4200002720202506190965448507",
                "4200002647202506196599952449",
                "4200002730202506184844995094",
                "4200002653202506183863644113",
                "4200002652202506180186949103",
                "4200002659202506183623841919",
                "4200002740202506172644740642",
                "4200002742202506178743555065",
                "4200002732202506171267939585",
                "4200002651202506175123791558",
                "4200002657202506170611506263",
                "4200002647202506171301431312",
                "4200002637202506173737816365",
                "4200002738202506171968866663",
                "4200002659202506165656184481",
                "4200002717202506169663432767",
                "4200002650202506161172685391",
                "4200002659202506164966156440",
                "4200002734202506164775501452",
                "4200002740202506160769069776",
                "4200002722202506163042719744",
                "4200002649202506161824273813",
                "4200002726202506165301273935",
                "4200002657202506165156481155",
                "4200002735202506166987141135",
                "4200002719202506167087266893",
                "4200002744202506160008840176",
                "4200002655202506167773681232",
                "4200002734202506169209923092",
                "4200002635202506164201341642",
                "4200002737202506165193916541",
                "4200002649202506161578648661",
                "4200002638202506156849499130",
                "4200002649202506141874130867",
                "4200002724202506145728385590",
                "4200002728202506141922478064",
                "4200002648202506148585209847",
                "4200002723202506130920024825",
                "4200002718202506135916678677",
                "4200002643202506134852730012",
                "4200002660202506137636545976",
                "4200002643202506135753987653",
                "4200002720202506131675723765",
                "4200002656202506138717903144",
                "4200002634202506135733701457",
                "4200002731202506139167503493",
                "4200002643202506133443744265",
                "4200002725202506135997387574",
                "4200002666202506139199351639",
                "4200002636202506133711103487",
                "4200002664202506132638393649",
                "4200002728202506125152607273",
                "4200002731202506121791552804",
                "4200002736202506110603029380",
                "4200002740202506110249500157",
                "4200002663202506108818116257",
                "4200002724202506095562739311",
                "4200002637202506090652296466",
                "4200002647202506092391012025",
                "4200002666202506091720900852",
                "4200002635202506088959312800",
                "4200002650202506080639425729",
                "4200002720202506089163486252",
                "4200002637202506084155306477",
                "4200002730202506077484183210",
                "4200002735202506070941030926",
                "4200002662202506067276982964",
                "4200002644202506031196471261",
                "4200002720202505318926638602",
                "4200002731202505297469535491",
                "4200002667202505296263452797",
                "4200002668202505284960560898",
                "4200002719202505236963549382",
                "4200002636202505232145087500",
                "4200002743202505239292557118",
                "4200002730202505235177423336",
                "4200002734202505233517097589",
                "4200002720202505237107093665",
                "4200002735202505221011471952",
                "4200002728202505161124147502",
                "4200002651202505129141813092",
                "4200002641202505109484259131",
                "4200002723202505096888478533",
                "4200002633202505061827450653",
                "4200002618202505061849718677",
                "4200002658202505054041839873",
                "4200002607202505059650896891",
                "4200002603202505043338678460",
                "4200002613202505044879711568",
                "4200002646202505036983340019",
                "4200002634202505019722463479"
        );
        List<FsStorePayment> fsStorePaymentList = fsStorePaymentService.queryByBankTrxId(bankTrxIds);
        for (FsStorePayment fsStorePayment : fsStorePaymentList) {
            FsStoreOrder order = fsStoreOrderMapper.selectFsStoreOrderById(fsStorePayment.getOrderId());
            List<FsStoreDelivers> byOrderId = fsStoreDeliversMapper.findByOrderId(order.getId());
            if(CollectionUtils.isEmpty(byOrderId)) {
                log.info("该单无物流信息 单号:{}",order.getId());
                continue;
            }
            FsStoreDelivers delivers = byOrderId.get(0);

            FsWxExpressTask fsWxExpressTask = new FsWxExpressTask();
            fsWxExpressTask.setUserId(order.getUserId());
            fsWxExpressTask.setStatus(0);
            fsWxExpressTask.setRetryCount(0);
            fsWxExpressTask.setCreateTime(LocalDateTime.now());
            fsWxExpressTask.setUpdateTime(LocalDateTime.now());
            fsWxExpressTask.setOrderCode(order.getOrderCode());
            fsWxExpressTask.setExpressCompany(delivers.getDeliverSn());
            fsWxExpressTask.setExpressNo(delivers.getDeliverId());
            fsWxExpressTask.setPayCode(fsStorePayment.getPayCode());

            try {
                fsWxExpressTaskMapper.insert(fsWxExpressTask);
            }catch (Exception e){
                log.error("插入失败",e);
            }
        }

    }
    @Test
    public void testTestPaymentSync() {
        fsStorePaymentService.paymentSync();
    }

    /**
     * 已支付 但是没有银行流水号数据修复
     */
    @Test
    public void updatePaymentsBankInfo(){
        String[] payCodeList = new String[]{
                "1899282845931667456"
        };

        for (String payCode : payCodeList) {
            FsStorePayment fsStorePayment = fsStorePaymentService.selectFsStorePaymentByCode(payCode);
            if(fsStorePayment != null){
                fsStorePaymentService.updateYbBankSerial(fsStorePayment);
            }
        }
    }

    @Autowired
    private MiniProgramSubTask miniProgramSubTask;
    @Test
    public void returnPayStatus() throws Exception {

        Authentication authentication = new UsernamePasswordAuthenticationToken(
                "admin",
                "sadmin#258",
                Collections.singletonList(new SimpleGrantedAuthority("ADMIN"))
        );

        // Set the Authentication in the SecurityContext
        SecurityContextHolder.getContext().setAuthentication(authentication);


        FsStorePayment fsStorePayment = new FsStorePayment();
        fsStorePayment.setPaymentId(01L);
        fsStorePayment.setStatus(1);
        fsStorePayment.setOrderId(1899279950347763712L);
        fsStorePayment.setPayTypeCode("weixin");
        fsStorePayment.setPayMoney(new BigDecimal("0.01"));
        fsStorePayment.setPayTime(new Date());
        fsStorePayment.setTradeNo("002900TOP3B250311101453P779ac1369bc00000");
        fsStorePayment.setUserId(520391L);
        fsStorePayment.setOpenId("obmfC6y5ZOuvq9b4UlL7GUayuYQE");
        fsStorePayment.setCompanyId(12L);
        fsStorePayment.setCompanyUserId(42L);
        fsStorePayment.setDeptId(144L);
        fsStorePayment.setBankSerialNo("1899279950347763712L");
        fsStorePayment.setPayMode("hf");
        R r = fsStorePaymentController.returnPayStatus(fsStorePayment);
    }

    @Test
    public void test() throws Exception {
        FsCouponSchedule fsCouponSchedule = new FsCouponSchedule();
        fsCouponSchedule.setOrderId(1899279950347763712L);
        fsCouponSchedule.setSetmealId(1800L);
        fsCouponSchedule.setSetmealTitle("甘草干姜汤（北京同仁堂）1盒");
        fsCouponSchedule.setMonth(4L);
        fsCouponSchedule.setCount(0L);
        fsCouponSchedule.setStatus(IcgProcessStatusEnum.PENDING.getCode());
        fsCouponSchedule.setOrderTime(LocalDateTime.now());
        fsCouponSchedule.setUserId(520391L);
        // 如果上次发送时间为空，那么预计发送时间等于=下单时间+1月
        fsCouponSchedule.setSendTime(fsCouponSchedule.getOrderTime().plusMonths(1L));
        fsCouponSchedule.setRetryCount(0);

        fsCouponScheduleService.insertFsCouponSchedule(fsCouponSchedule);
    }

    @Autowired
    private TzBankService tzBankService;

    @Autowired
    private YbPaymentHandler ybPaymentHandler;

    @Autowired
    private HfPaymentHandler hfPaymentHandler;

    @Autowired
    private ISysConfigService configService;
    @Test
    public void ybRefund(){
        PayProcessContext processContext = new PayProcessContext();
        processContext.setOrderId(null);
        processContext.setPayType(null);
        processContext.setPaymentId(1334828L);
        processContext.setPayCode("1929745734509264897");
        String json = configService.selectConfigByKey("store.pay");
        FsPayConfig fsPayConfig = JSON.parseObject(json, FsPayConfig.class);
        processContext.setFsPayConfig(fsPayConfig);
        processContext.setUserId(null);
        processContext.setGoodsInfo("商城订单支付");
        processContext.setOrderPrefix("store-");
        ybPaymentHandler.refundOrder(processContext);
    }


    @Test
    public void hfRefund(){
        PayProcessContext processContext = new PayProcessContext();
        processContext.setOrderId(null);
        processContext.setPayType(null);
        processContext.setPaymentId(1334828L);
        processContext.setPayCode("1947198650681655296");
        String json = configService.selectConfigByKey("store.pay");
        FsPayConfig fsPayConfig = JSON.parseObject(json, FsPayConfig.class);
        processContext.setFsPayConfig(fsPayConfig);
        processContext.setUserId(null);
        processContext.setGoodsInfo("商城订单支付");
        processContext.setOrderPrefix("store-");
        hfPaymentHandler.refundOrder(processContext);
    }

    /**
     * 手动退款
     */
    @Test
    public void refund(){
        String[] data = new String[]{
                "1947198650681655296"
        };
        TzConfigInfoDTO tzConfigInfoDTO = TzConfigUtils.getConfig();

        for (String datum : data) {
            FsStorePayment payment = fsStorePaymentService.selectFsStorePaymentByCode(datum);

            RequestDTO<TzPayRefundDTO> requestDTO = new RequestDTO<TzPayRefundDTO>();
            TzPayRefundDTO tzPayRefundDTO = new TzPayRefundDTO();
            tzPayRefundDTO.setOldPayOutOrderNo(payment.getPayCode());
            tzPayRefundDTO.setRefundOrderNo(payment.getPayCode()+"1");
            tzPayRefundDTO.setPlatMerCstNo(tzConfigInfoDTO.getPlatMerCstNo());
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
            tzPayRefundDTO.setTrxSendTime(sdf.format(new Date()));
            tzPayRefundDTO.setRefundOrdTransAmt(payment.getPayMoney().subtract(BigDecimal.valueOf(0.1)));
            tzPayRefundDTO.setNotifyURL(tzConfigInfoDTO.getRefundNotifyUrl());
            requestDTO.setReqBody(tzPayRefundDTO);
            requestDTO.setReqHeader(TzReqHeaderDTO.getInstance(payment.getPayCode()));

            TzReqResultDTO<TzRefundNotifyDTO> resultDTO;

            try {

                resultDTO = tzBankService.payRefund(requestDTO);
                TzRefundNotifyDTO body = resultDTO.getBody();
                String refundOrdStatus = body.getRefundOrdStatus();
                log.info("退款支付单号:{} 返回结果：{}",datum, resultDTO);
                if("90".equals(refundOrdStatus) || "50".equals(refundOrdStatus)) {
                    payment.setRefundMoney(payment.getRefundMoney());
                    payment.setStatus(-1);
                    payment.setRefundTime(new Date());
                    fsStorePaymentService.updateFsStorePayment(payment);
                }
            } catch (Exception e) {
                throw new RuntimeException(e);
            } finally {

            }
        }
        // 退款将定时发放优惠券的状态设置为已退款
//                fsCouponScheduleService.setScheduleCouponRefund(String.valueOf(payment.getOrderId()));

    }


    @Autowired
    private ExpressTask expressTask;

    @Test
    public void deliveryOp(){
        storeTask.deliveryOp();
    }

    @Test
    public void subscribeExpress(){
        expressTask.subscribeExpress();
    }

    @Test
    public void issueCoupon() throws Exception {
        fsCouponScheduleService.issueCoupon();
    }

    @Test
    public void notifyMiniAppSub(){
        miniProgramSubTask.notifyMiniAppSub();
    }

    @Test
    public void queryOrder() throws Exception {
        String payCode = "1909873348586766336";
        FsStorePayment payment = fsStorePaymentService.selectFsStorePaymentByCode(payCode);

        RequestDTO<QueryOrderRestDTO> requestDTO = new RequestDTO<>();

        QueryOrderRestDTO queryOrderRestDTO = new QueryOrderRestDTO();
        queryOrderRestDTO.setOrderFlowNo(payment.getTradeNo());
        TzConfigInfoDTO tzConfigInfoDTO = TzConfigUtils.getConfig();

        queryOrderRestDTO.setPlatMerCstNo(tzConfigInfoDTO.getPlatMerCstNo());

        requestDTO.setReqBody(queryOrderRestDTO);
        requestDTO.setReqHeader(TzReqHeaderDTO.getInstance(payment.getPayCode()));

        TzReqResultDTO<QueryOrderInfoDTO> resultDTO = tzBankService.payQueryOrder(requestDTO);
        log.info("查询结果: {}",resultDTO);
        String retCode = resultDTO.getRetCode();
        if(!retCode.equals("00000000")){
            log.info("台州银行支付回调查询失败{}",resultDTO);
            throw new CustomException("台州银行支付回调查询失败! msg="+resultDTO.getRetCode());
        }
        QueryOrderInfoDTO body = resultDTO.getBody();

    }
}
